/******************************************************************************
 * 
 * Copyright 2012 - 2013 Vektor Software.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *****************************************************************************/


package com.vektorsoft.demux.android.core.app;

import android.app.Activity;
import android.view.View;
import com.vektorsoft.demux.core.mva.AbstractViewManager;
import com.vektorsoft.demux.core.mva.DMXAbstractView;
import com.vektorsoft.demux.core.mva.DMXAdapter;
import com.vektorsoft.demux.core.mva.DMXView;
import java.util.Collections;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * Implementation of {@link DMXViewManager} for Android platform. This implementation should 
 * be able to handle registering views with activities in a manner consistent with the rest of
 * the framework.
 *
 * @author Vladimir Djurovic
 */
public class DMXAndroidViewManager extends AbstractViewManager {
    
    /** Name of the root view. */
    public static final String DMX_ANDROID_ROOT_VIEW = "com.vektorsoft.demux.android.core.DMXAndroidViewManager.root";
    
    /** Application main activity. */
    private Activity mainActivity;

    /**
     * Creates new instance.
     * 
     * @param activity  main activity
     */
    public DMXAndroidViewManager(Activity activity, DMXAdapter adapter){
        this.mainActivity = activity;
        this.adapter = adapter;
        DMXAndroidRootView rootView = new DMXAndroidRootView(mainActivity);
        registerView(rootView);
    }

    @Override
    protected void processDeferredViews(final String parentId, Set<String> childrenIds) {
         if(childrenIds != null && !childrenIds.isEmpty()){
            for(final String id : childrenIds){
                mainActivity.runOnUiThread(new Runnable() {

                    @Override
                    public void run() {
                        DMXView view = getView(id);
                        getView(parentId).addChildView(view);
                        view.render();
                    }
                });
            }
        }
    }

    @Override
    public void selectView(String viewId, Map<String, Object> dataValues) {
        
    }

    @Override
    public void updateView(String viewId, Map<String, Object> dataValues) {
        DMXView view = getView(viewId);
        view.updateFromModel(dataValues);
        view.render();
    }

    @Override
    public void updateViewData(final Map<String, Object> dataMap) {
         //find views that need to be refreshed
        final Set<String> viewIds = new HashSet<String>();
        for(String id : dataMap.keySet()){
            viewIds.addAll(getViewIdsForData(id));
        }
        // refresh views
        // should this all run in signle Runnable, or have separate Runnable for each view?
        mainActivity.runOnUiThread(new Runnable() {

            @Override
            public void run() {
                for(String id : viewIds){
                    updateView(id, dataMap);
                }
            }
        });
    }

    @Override
    public final void registerView(final DMXView view) {
        super.registerView(view);
        if(view instanceof DMXAndroidView){
            ((DMXAndroidView)view).setContext(mainActivity);
        }
        if(view instanceof DMXAbstractView){
            ((DMXAbstractView)view).setAdapter(adapter);
        }
        // add this view to it's parent
        final String parentId = view.getParentViewId();
        if(parentId != null && !parentId.isEmpty()){
            mainActivity.runOnUiThread(new Runnable() {

                @Override
                public void run() {
                    DMXView parent = getView(parentId);
                    parent.addChildView(view);
                    view.render();
                }
            });
            
        }
    }
    
    /**
     * Root view of Android application.
     */
    private static class DMXAndroidRootView implements DMXView {
        
        /** Application main activity. */
        private Activity mainActivity;

        /**
         * Creates new instance.
         * 
         * @param activity application main activity
         */
        public DMXAndroidRootView(Activity activity) {
            this.mainActivity = activity;
        }
        

        @Override
        public void render() {
            
        }

        @Override
        public String getViewId() {
            return DMX_ANDROID_ROOT_VIEW;
        }

        @Override
        public String getParentViewId() {
            return null;
        }

        @Override
        public Object viewPlacementConstraint() {
            return null;
        }

        @Override
        public void addChildView(DMXView child) {
            Object viewUi = child.getViewUI();
            if(!(viewUi instanceof View)){
                throw new IllegalArgumentException("View UI must be instance of android.view.View");
            }
            mainActivity.setContentView((View)viewUi);
        }

        @Override
        public Set<String> getViewDataIds() {
            return Collections.EMPTY_SET;
        }

        @Override
        public void updateFromModel(Map<String, Object> dataMap) {
        }

        @Override
        public Object getViewUI() {
            return mainActivity;
        }
        
    }

}
